'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var client = require('@apollo/client');
var fetch = require('cross-fetch');
var middleware = require('@vue-storefront/middleware');
var requestIp = require('request-ip');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fetch__default = /*#__PURE__*/_interopDefaultLegacy(fetch);
var requestIp__default = /*#__PURE__*/_interopDefaultLegacy(requestIp);

const buildClient = (settings) => {
    const httpLink = client.createHttpLink({
        uri: settings.odooGraphqlUrl,
        credentials: 'include',
        fetch: fetch__default["default"],
        fetchOptions: settings.fetchOptions,
        headers: {
            Cookie: settings.sessionAuth,
            'resquest-host': settings.requestHost,
            'REAL-IP': settings.realIp
        }
    });
    const afterwareLink = new client.ApolloLink((operation, forward) => {
        return forward(operation).map((response) => {
            const context = operation.getContext();
            const authHeader = context.response.headers.get('set-cookie');
            if (response.data) {
                response.data.cookie = authHeader;
            }
            return response;
        });
    });
    const apolloLink = client.ApolloLink.from([
        afterwareLink.concat(httpLink)
    ]);
    return new client.ApolloClient({
        link: apolloLink,
        cache: new client.InMemoryCache(),
        ssrMode: true,
        credentials: 'include',
        defaultOptions: {
            query: {
                errorPolicy: "all",
                fetchPolicy: "no-cache",
            },
            mutate: {
                errorPolicy: "all",
            }
        }
    });
};

const sessionHeaderExtension = {
    name: 'sessionHeaderExtension ',
    hooks: (req, res) => {
        return {
            beforeCreate: ({ configuration }) => {
                const clientIp = requestIp__default["default"].getClientIp(req);
                return {
                    ...configuration,
                    sessionAuth: req.headers.cookie,
                    requestHost: req.headers.host,
                    realIp: clientIp
                };
            },
            beforeCall: ({ configuration, callName, args }) => args,
            afterCall: ({ configuration, callName, response }) => {
                if (response?.data?.cookie) {
                    res.setHeader('Set-cookie', response?.data?.cookie);
                }
                return response;
            }
        };
    }
};

const mutation = async (context, metadata, params) => {
    //@ test queries
    const mutation = context.config.queries[metadata.mutationName];
    console.log(mutation);
    console.log(metadata);
    console.log(params);
    const response = await context.client.mutate({
        variables: params,
        mutation,
        fetchPolicy: 'no-cache',
        errorPolicy: 'all'
    });
    return response;
};

const query = async (context, metadata, params) => {
    const query = context.config.queries[metadata.queryName];
    const response = await context.client.query({
        variables: params,
        query,
        fetchPolicy: 'no-cache',
        errorPolicy: 'all'
    });
    return response;
};

var apiEndpoints = /*#__PURE__*/Object.freeze({
  __proto__: null,
  mutation: mutation,
  query: query
});

const onCreate = (settings) => {
    const client = buildClient(settings);
    return {
        config: settings,
        client
    };
};
const { createApiClient } = middleware.apiClientFactory({
    onCreate,
    api: apiEndpoints,
    extensions: [
        sessionHeaderExtension
    ]
});

exports.createApiClient = createApiClient;
//# sourceMappingURL=index.js.map
